/*************************************************************************
                      4-gon.h

***************************************************************************/
#include <matrix.h>
#include <debug-control.h>
#include <util.h>
#include <generic-code.h>



/* function object for comparing index values with 'find_if' */
struct vertex_equals
{
	int vertex;
	vertex_equals(int v): vertex(v) {}
	bool operator () (const int p) {return p == vertex;}
};


class region_data
{
	public: 
	
	int index;  // used for debugging only
	int num_boundary_vertices;
	int num_boundary_edges;  // a region may meet the boundary in vertices only
	int num_inside_vertices;  //requiring connection to an interior_vertex
	int num_interior_vertices;
	int first_interior_vertex;
	vector<int> boundary_vertices; // length num_boundary_vertices
	vector<int> inside_vertices;  // length num_inside_vertices
	vector<int> interior_choice;  // length num_inside_vertices
	vector<int> max_interior_choice;  // length num_inside_vertices
	
	region_data():index(0),num_boundary_vertices(0),num_inside_vertices(0),num_interior_vertices(0),first_interior_vertex(0){}

};

class chord_data
{
	public:		
	
	int num_edges_to_add;  // number of additional interior edges required by boundary chords and inside vertices
	int total_num_inside_vertices;  // note: inside, not interior
	matrix<int> incidence; // records the boundary and boundary chord edge incidence data
	list<pair<int,int> > chords;
	vector<int> min_num_interior_vertices; // length chords.size()+1
	vector<region_data> region; // length chords.size()+1		
	vector<int> region_map; // length num_boundary_vertices region_map[i] records the region incident with the edge from boundary vertex i to i+1 (mod 2p)
	chord_data(): num_edges_to_add(0), total_num_inside_vertices(0) {}
	chord_data(matrix<int> I, list<pair<int,int> > l):num_edges_to_add(0), total_num_inside_vertices(0), incidence(I), chords(l){} 
	void push(pair<int,int> p) {chords.push_back(p);}
};

class result_data
{
	public:
	
	vector<int> doodle_code;
	int num_boundary_vertices;
	int solution_count;
	list<string> doodle_list;
	
	result_data(vector<int> code, int n): doodle_code(code),num_boundary_vertices(n),solution_count(0) {}
};

void write_region_data(region_data& region,ostream& os, string prefix);
void write_chord_data(chord_data& pattern,ostream& os, string prefix);
