#include <ogdf/fileformats/GraphIO.h>
#include <ogdf/orthogonal/OrthoLayout.h>
#include <ogdf/planarity/EmbedderMinDepthMaxFaceLayers.h>
#include <ogdf/planarity/PlanarSubgraphFast.h>
#include <ogdf/planarity/PlanarizationLayout.h>
#include <ogdf/planarity/SubgraphPlanarizer.h>
#include <ogdf/planarity/VariableEmbeddingInserter.h>


#include <ogdf/layered/MedianHeuristic.h>
#include <ogdf/layered/OptimalHierarchyLayout.h>
#include <ogdf/layered/OptimalRanking.h>
#include <ogdf/layered/SugiyamaLayout.h>
#include <ogdf/energybased/DavidsonHarelLayout.h>
#include <ogdf/energybased/TutteLayout.h>


using namespace std; 
using namespace ogdf;
 
int main()
{

/*
    Graph G;    
    GraphAttributes GA(G,
      GraphAttributes::nodeGraphics | GraphAttributes::nodeType |
      GraphAttributes::edgeGraphics | GraphAttributes::edgeType);
 
    if (!GraphIO::read(GA, G, "ERDiagram.gml", GraphIO::readGML)) {
        std::cerr << "Could not read ERDiagram.gml" << std::endl;
        return 1;
    }
 
    for (node v : G.nodes)
    {
        GA.width(v) /= 2;
        GA.height(v) /= 2;
    }
 
    PlanarizationLayout pl;
 
    SubgraphPlanarizer *crossMin = new SubgraphPlanarizer;
    PlanarSubgraphFast<int> *ps = new PlanarSubgraphFast<int>;
    ps->runs(100);
    VariableEmbeddingInserter *ves = new VariableEmbeddingInserter;
    ves->removeReinsert(RemoveReinsertType::All);
 
    crossMin->setSubgraph(ps);
    crossMin->setInserter(ves);
    pl.setCrossMin(crossMin);
 
    EmbedderMinDepthMaxFaceLayers *emb = new EmbedderMinDepthMaxFaceLayers;
    pl.setEmbedder(emb);
 
    OrthoLayout *ol = new OrthoLayout;
    ol->separation(20.0);
    ol->cOverhang(0.4);
    pl.setPlanarLayouter(ol);
 
    pl.call(GA);
 
    GraphIO::write(GA, "output-ERDiagram.gml", GraphIO::writeGML);
    GraphIO::write(GA, "output-ERDiagram.svg", GraphIO::drawSVG);
*/


    Graph G;    

	/* test of a disconnected graph */
	string DOT = "graph G {0--1--2--3--4--5--6--7--0 0--8 2--8 4--8 6--8  9--10--11--12--9 13--14--15--16--13 9--13 10--14 11--15 12--16}";

//	string DOT = "graph G {0--1--2--3--4--5--6--7--0 0--8 0--10 2--8 4--8 4--11 5--12 6--9 7--13 9--13--10--14--11--12--9 9--14--8}";

	istringstream iss(DOT);
	
	ifstream input("March_test.dot");
	
    GraphAttributes GA(G, GraphAttributes::nodeGraphics | 
                          GraphAttributes::nodeType | 
                          GraphAttributes::edgeGraphics | 
                          GraphAttributes::edgeType);
	  
	if (GraphIO::readDOT(GA, G, iss))
//	if (GraphIO::readDOT(GA, G, input))
	{

		bool planar = ogdf::isPlanar(G);
		cout <<  "planar graph = " << boolalpha << planar << endl;
		
/******************** rectangular planarization *******************/
	   for (node v : G.nodes)
	    {
	        GA.width(v) /= 2;
	        GA.height(v) /= 2;
	    }

	    PlanarizationLayout pl;
	    pl.call(GA);
	 		
	    GraphIO::write(GA, "__planar_graph.svg", GraphIO::drawSVG); // used for debugging only
	    GraphIO::write(GA, "__planar_graph.dot", GraphIO::writeDOT);
/********************************************************************/	    


/******************** Davidson Harel *******************/
	    DavidsonHarelLayout dh;
	    dh.call(GA);
	 		
	    GraphIO::write(GA, "__Davidson_Harel.svg", GraphIO::drawSVG); // used for debugging only
	    GraphIO::write(GA, "__Davidson_Harel.dot", GraphIO::writeDOT);
/********************************************************************/	    

/******************** tUTTE *******************/
	    TutteLayout tl;
	    tl.call(GA);
	 		
	    GraphIO::write(GA, "__Tutte.svg", GraphIO::drawSVG); // used for debugging only
	    GraphIO::write(GA, "__Tutte.dot", GraphIO::writeDOT);
/********************************************************************/	    

/************************ SUGIYAMA ************************************/

    SugiyamaLayout SL;
    SL.setRanking(new OptimalRanking);
    SL.setCrossMin(new MedianHeuristic);
 
    OptimalHierarchyLayout *ohl = new OptimalHierarchyLayout;
    ohl->layerDistance(30.0);
    ohl->nodeDistance(25.0);
    ohl->weightBalancing(0.8);
    SL.setLayout(ohl);
 
    SL.call(GA);
    GraphIO::write(GA, "output-Sugiyama.gml", GraphIO::writeGML);
    GraphIO::write(GA, "output-Sugiyama.svg", GraphIO::drawSVG);
/******************************************************************/
 
	}
	else
	{
		cerr << "Error, could not read graph from DOT data" << endl;
		exit(0);
	}
 
    return 0;
}
